"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubmitSidechainCrossChainUpdateCommand = void 0;
const state_machine_1 = require("../../../../state_machine");
const base_cross_chain_update_command_1 = require("../../base_cross_chain_update_command");
const constants_1 = require("../../constants");
const chain_account_1 = require("../../stores/chain_account");
const chain_validators_1 = require("../../stores/chain_validators");
const utils_1 = require("../../utils");
class SubmitSidechainCrossChainUpdateCommand extends base_cross_chain_update_command_1.BaseCrossChainUpdateCommand {
    async verify(context) {
        const { params } = context;
        if (params.certificate.length === 0 && (0, utils_1.isInboxUpdateEmpty)(params.inboxUpdate)) {
            throw new Error('A cross-chain update must contain a non-empty certificate and/or a non-empty inbox update.');
        }
        if (!params.sendingChainID.equals((0, utils_1.getMainchainID)(context.chainID))) {
            throw new Error('Only the mainchain can send a sidechain cross-chain update.');
        }
        const sendingChainExist = await this.stores
            .get(chain_account_1.ChainAccountStore)
            .has(context, params.sendingChainID);
        if (!sendingChainExist) {
            throw new Error('The mainchain is not registered.');
        }
        const isLive = await this.internalMethod.isLive(context, params.sendingChainID);
        if (!isLive) {
            throw new Error('The sending chain is not live.');
        }
        const sendingChainAccount = await this.stores
            .get(chain_account_1.ChainAccountStore)
            .get(context, params.sendingChainID);
        if (sendingChainAccount.status === 0 && params.certificate.length === 0) {
            throw new Error('Cross-chain updates from chains with status CHAIN_STATUS_REGISTERED must contain a non-empty certificate.');
        }
        if (params.certificate.length > 0) {
            await this.internalMethod.verifyCertificate(context, params, context.header.timestamp);
        }
        const sendingChainValidators = await this.stores
            .get(chain_validators_1.ChainValidatorsStore)
            .get(context, params.sendingChainID);
        if (!(0, utils_1.emptyActiveValidatorsUpdate)(params.activeValidatorsUpdate) ||
            params.certificateThreshold !== sendingChainValidators.certificateThreshold) {
            await this.internalMethod.verifyValidatorsUpdate(context, params);
        }
        if (!(0, utils_1.isInboxUpdateEmpty)(params.inboxUpdate)) {
            await this.internalMethod.verifyPartnerChainOutboxRoot(context, params);
        }
        return {
            status: state_machine_1.VerifyStatus.OK,
        };
    }
    async execute(context) {
        const [decodedCCMs, ok] = await this.executeCommon(context, false);
        if (!ok) {
            return;
        }
        const { params } = context;
        try {
            context.contextStore.set(constants_1.CONTEXT_STORE_KEY_CCM_PROCESSING, true);
            for (let i = 0; i < decodedCCMs.length; i += 1) {
                const ccm = decodedCCMs[i];
                const ccmBytes = params.inboxUpdate.crossChainMessages[i];
                const ccmContext = {
                    ...context,
                    ccm,
                    ccu: {
                        sendingChainID: params.sendingChainID,
                    },
                };
                await this.apply(ccmContext);
                await this.internalMethod.appendToInboxTree(context, params.sendingChainID, ccmBytes);
            }
        }
        finally {
            context.contextStore.delete(constants_1.CONTEXT_STORE_KEY_CCM_PROCESSING);
        }
        await this.afterExecuteCommon(context);
    }
}
exports.SubmitSidechainCrossChainUpdateCommand = SubmitSidechainCrossChainUpdateCommand;
//# sourceMappingURL=submit_sidechain_cross_chain_update.js.map